##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
COM Support for Zope

To use this Product, you need to have Python win32 extensions
installed. See http://www.python.org/windows/

"""
__version__='$Revision: 1.00 $'[11:-2]

from Globals import Persistent, HTMLFile
import Acquisition
import OFS.SimpleItem
import AccessControl.Role
import win32com.client

addObjectForm=HTMLFile('objectAddForm', globals())

def addObject(self, id, title, dispatch, REQUEST=None):
    """
    Create a COM Object.
    """
    i=COMObject(id,title,dispatch)
    self._setObject(id,i)
    if REQUEST is not None:
        return self.manage_main(self,REQUEST)


class COMObject(OFS.SimpleItem.Item, Persistent, Acquisition.Explicit,
        AccessControl.Role.RoleManager):
    """
    Gives Zope access to a COM object.
    """
    
    # Acquire ZopeTime
    ZopeTime=Acquisition.Acquired
    
    meta_type="COM Object"
    
    manage_options=(
        {'label':'Properties', 'action':'manage_main'},
        {'label':'Connection', 'action':'manage_connect'},
        {'label':'Security', 'action':'manage_access'},
        )

    __ac_permissions__=(
        ('View management screens',
                ('manage_main','manage_connect','connected')),
        ('Change COM Objects', ('manage_edit','connect','disconnect')),
        ('Use COM Objects', ('__call__','constants','setattr')),
        )

    def __init__(self, id, title, dispatch):
        self.id=id
        self.manage_edit(title,dispatch)
    
    manage_main=HTMLFile('objectEditForm', globals())
    manage_connect=HTMLFile('objectConnectForm', globals())
    
    def manage_edit(self, title, dispatch, REQUEST=None):
        """
        Edit the COM Object.
        
            'title' -- The title of the Object.
            
            'dispatch' -- The dispatch name of the COM object.
        """
        self.title=title
        self.dispatch=dispatch
        self.disconnect()
        if REQUEST is not None:
            return self.manage_main(self, REQUEST,
                manage_tabs_message="Changed."
                )
    
    def __call__(self):
        """
        Access the COM object. You cannot access the COM object directly,
        you must call the Zope object to get access to the COM object.
        """
        if not self.connected():
            self.connect()
        return self._v_o

    def constants(self,key):
        """
        Access to COM constants.
        """
        return getattr(win32.client.constants,key)
    
    def setattr(self,key,value):
        """
        Set an attribute on the COM object.
        """
        if not self.connected():
            self.connect()
        setattr(self._v_o, key, value)
    
    def connected(self):
        """
        Returns true if we are currently connected to a COM Object.
        """
        if hasattr(self, '_v_o'):
            return 1
    
    def disconnect(self,REQUEST=None):
        """
        Disconnect from the COM object.
        """
        if self.connected():
            del self._v_o
        if REQUEST is not None:
            return self.manage_connect(self,REQUEST,
                    manage_tabs_message="Disconnected.")
                    
    def connect(self,REQUEST=None):
        """
        Connect to the COM object.
        """
        if not self.connected():
            self._v_o=self.getObject()
        if REQUEST is not None:
            return self.manage_connect(self,REQUEST,
                    manage_tabs_message="Connected.")
        
    def getObject(self):
        try:
            o=win32com.client.Dispatch(self.dispatch)
            self._v_o=o
            return o
        except:
            self.raise_standardErrorMessage() 
            
        
        